--[[
	Mailbox Plus: Improved Mailbox
	Speeddymon, aka Speeddy, aka Thomas Spear <Speeddymon@gmail.com>
]]--

MailboxPlus = LibStub("AceAddon-3.0"):NewAddon("MailboxPlus","AceEvent-3.0","AceHook-3.0","AceConsole-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("MailboxPlus", true)

local options = {
	type = 'group',
	name = "MailboxPlus",
	handler = MailboxPlus,
	args = {
		sending = {
			type = "group",
			order = 1,
			name = L["Sending Options"],
			desc = L["Options for the mail sending window."],
			args = {
				autosend = {
					type = "toggle",
					order = 1,
					name = L["Autosend"],
					desc = L["Instantly send mail when you have attached the maximum number of items per mail."],
					set = function() MailboxPlus.db.profile.autosend = not MailboxPlus.db.profile.autosend end,
					get = function() return MailboxPlus.db.profile.autosend end
				},
				listsize = {
					type = "range",
					order = 2,
					name = L["Dropdown size"],
					desc = L["Size of the dropdown list"],
					min = 1,
					max = 30,
					set = function() MailboxPlus.db.profile.listsize = not MailboxPlus.db.profile.listsize end,
					get = function() return MailboxPlus.db.profile.listsize end
				},
				rememberalts = {
					type = "toggle",
					order = 3,
					name = L["Remember Alts"],
					desc = L["Remember your own Alts in the dropdown when sending mail to them. Blizzard's autocomplete will work even with this disabled"],
					set = function() MailboxPlus.db.profile.rememberalts = not MailboxPlus.db.profile.rememberalts end,
					get = function() return MailboxPlus.db.profile.rememberalts end
				},
			},
		},
		report = {
			type = "group",
			order = 2,
			name = L["Reporting Options"],
			desc = L["Delivery Reporting Options"],
			args = {
				showreport = {
					type = "execute",
					order = 1,
					name = L["Delivery Report"],
					desc = L["List mails and estimated delivery times."],
					func = nil -- TODO: Add a function to report mail deliveries
				},
				break1 = {
					type = "header",
					name = "",
					order = 2
				},
				reportonsend = {
					type = "toggle",
					order = 3,
					name = L["Report on Send"],
					desc = L["Report estimated mail delivery times when sending."],
					set =  function() MailboxPlus.db.profile.reportonsend = not MailboxPlus.db.profile.reportonsend end,
					get = function() return MailboxPlus.db.profile.reportonsend end
				},
				reportforalts = {
					type = "toggle",
					order = 4,
					name = L["For Alts"],
					desc = L["Also report estimated mail delivery times for your alts when sending."],
					set = function() MailboxPlus.db.profile.reportforalts = not MailboxPlus.db.profile.reportforalts end,
					get = function() return MailboxPlus.db.profile.reportforalts end
				},
			},
		},
	},
}

--Default settings
local defaults = {
	profile = {
		autosend = false,
		reportonsend = true,
		reportforalts = true,
		listsize = 10,
		rememberalts = true,
	},
	factionrealm = {
		lastmailedto = nil,
		alts = {},
		recipients = {}
	},
	char = {
		intransit = {},
	}
}

function MailboxPlus:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("MailboxPlusDB")
	self.db:RegisterDefaults(defaults)

	LibStub("AceConfig-3.0"):RegisterOptionsTable("MailboxPlus", options, {"mbp", "mbplus", "mailboxplus"})
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("MailboxPlus", "MailboxPlus")
	self:RegisterChatCommand("mbp", "ChatSlash")
	self:RegisterChatCommand("mbplus", "ChatSlash")
	self:RegisterChatCommand("mailboxplus", "ChatSlash")
end

function MailboxPlus:OnEnable()
	-- Add current character to alt list
	if not self:Find((UnitName("player")), self.db.factionrealm.alts) then
		table.insert(self.db.factionrealm.alts, (UnitName("player")))
	end

	table.sort(self.db.factionrealm.alts)

	self:Hook("SendMailFrame_SendMail", "SendMail", true)

	self:HookScript(SendMailNameEditBox, "OnEditFocusGained", "AutoFill")
end

function MailboxPlus:OnDisable()
--
end

-- Slash command handler
function MailboxPlus:ChatSlash(arg)
	if not arg or arg:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame) --Opens the Blizz options frame
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(MailboxPlus, "mbp", "MailboxPlus", arg) --Parses arguments to slash commands in the chat window
	end
end

local function IsAlt(character)
	return self:Find(character, self.db.factionrealm.alts)
end

-- Find given character name in various tables (searchloc), return the index in the table of that name if found
function MailboxPlus:Find(character, searchloc)
	for index, name in pairs(searchloc) do
        if name:lower() == character:lower() then
			return index
		end
	end
	return nil
end

-- The dropdown menu itself
function MailboxPlus:DropdownInit()

	-- Index of the recipient in the recipients table
	local idx = MailboxPlus:Find(SendMailNameEditBox:GetText(), MailboxPlus.db.factionrealm.recipients)

	-- Create and populate the dropdown menu
	local button
	for index, name in ipairs(MailboxPlus.db.factionrealm.recipients) do
		if index > MailboxPlus.db.profile.listsize then break end
		if name ~= (UnitName("player")) then
			button = { text = name, value = index, func = MailboxPlus.NameSelect }
			button.checked = index == idx and 1 or nil
            UIDropDownMenu_AddButton(button)
		end
	end

	-- Remove button
	if SendMailNameEditBox:GetText() ~= "" then
		button = { notCheckable = 1 }
		if idx then
			button.text = L["[Remove %s]"]:format(SendMailNameEditBox:GetText())
			button.value = idx
			button.func = function()
				table.remove(MailboxPlus.db.factionrealm.recipients, button.value)
				SendMailNameEditBox:SetText("")
				SendMailNameEditBox:SetFocus()
			end
		end
		UIDropDownMenu_AddButton(button)
	end

end

-- Show the dropdown menu button on the send mail window
function MailboxPlus:DropdownShow()
	this.tooltip = L["Select/Add Recipient"]
	UIDropDownMenu_Initialize(this:GetParent(), self.DropdownInit)
	UIDropDownMenu_SetAnchor(this:GetParent(), 0, 0, "TOPRIGHT", 
							 this:GetName(), "BOTTOMRIGHT")
end

-- Hook for SendMail()
function MailboxPlus:SendMail()

	-- Ensure we aren't sending to ourself
	if (SendMailNameEditBox:GetText() == (UnitName("player"))) then
	    self.hooks["SendMailFrame_SendMail"]()
		return
	end

	-- Add recipient to table of recipients
	if (not self:Find(SendMailNameEditBox:GetText(), self.db.factionrealm.recipients)) or self:Find(SendMailNameEditBox:GetText(), self.db.factionrealm.alts) and self.db.profile.rememberalts then
		table.insert(self.db.factionrealm.recipients, SendMailNameEditBox:GetText())
	end

	table.sort(self.db.factionrealm.recipients)
	
	self.db.factionrealm.lastmailedto = SendMailNameEditBox:GetText()
	SendMailNameEditBox:ClearFocus()
	
	if self.db.profile.reportonsend then
		if not IsAlt(SendMailNameEditBox:GetText()) then
			self:Print(L["Mail '%s' will be delivered to %s in %d minutes or less"]:format(SendMailSubjectEditBox:GetText(), SendMailNameEditBox:GetText(), 62))
		elseif self.db.profile.reportforalts then
			self:Print(L["Mail '%s' will be delivered to %s in %d minutes or less"]:format(SendMailSubjectEditBox:GetText(), SendMailNameEditBox:GetText(), 2))
		end
	end

end

-- Function to add name to the edit box when selected from the menu
function MailboxPlus:NameSelect()
	local name = MailboxPlus.db.factionrealm.recipients[this.value]
	SendMailNameEditBox:SetText(name)
	SendMailSubjectEditBox:SetFocus()
end


function MailboxPlus:AutoFill()
	if SendMailNameEditBox:GetText() == "" and self.db.factionrealm.lastmailedto and 
		self.db.factionrealm.lastmailedto ~= me then

		SendMailNameEditBox:SetText(self.db.factionrealm.lastmailedto)
		-- Most of the time folks are just mailing items around, with
		-- Subject autofilled and no body, so might as well just clear
		-- the focus back out to the outer UI.
		SendMailNameEditBox:ClearFocus()
	else
		-- Highlight whatever (if anything) is there for possible replacement.
		SendMailNameEditBox:HighlightText(0,-1)
	end

	-- Call original script.
	self.hooks[SendMailNameEditBox].OnEditFocusGained()
end
